(ns com.aurellem.exp.music
  "scratchpad namespace"
  (:use (com.aurellem.gb gb-driver util constants
                         assembly saves
                         items pokemon status types
                         characters species moves
                         pokedex money rival-name
                         text-speed badges
                         pokemon-presets
                         ))
  
  (:import java.io.File)
  (:import [com.aurellem.gb.gb_driver SaveState]))




(defn analyze-music []
  (clojure.pprint/pprint
   (map (fn [[addr nums]]
          [(hex addr) nums])
        (apply harmonic-compare
               (map read-state
                    ["song-1" "no-song-1"
                     "song-2" "no-song-2"
                     "song-3" "no-song-3"
                     "song-4"])))))
;; there are only two addresses that alternate between only
;; two unique values:

0xC0DE ;; lol code :)
0xC0DF

;; perhaps they are the address of the current playing sound?

;; after watching them for a while in different environments,
;; I can tell that they're definately music related, but they
;; _don't_ seem to be the address to the current track.
;; they generally change whenever a substantually different
;; sequence is played.  maybe they control some high level
;; setting in the synthesizer? They are only affected by the
;; background music and not by pokemon cries, etc.

0xC001 ;; seems to change along with the music, but is much
;; more stable than 0xC0DE and 0xC0DF. It _is_
;; affected by sound effects.  When a sound effect happens,
;; is sometimes does not return to its previous value when the sound
;; effect is done.

0xC000 ;; does not seem to be affected by music

;; maybe 0xC001 - something is loaded with the actual sounds?


;; replacing just the first 0xFF bytes starting at 0xC001 with those
;; found in celadon changes the music to celadon city.

;; this can't be the actual music because it doesn't have enough
;; information.

;; let's try to narrow down the bytes from the first 0xFF of celadon
;; starting at 0xC001

(def celadon-song
  [202 0 0 255 0 178 117 40 118 158 118 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0
   0 0 0 0 0 0 0 0 0 0 0 202 202 202 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0
   0 0 0 0 128 128 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0
   0 0 0 0 0 0 0 0 0 0 0 0 115 88 32 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0
   0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0
   0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 5 5 5 1 1 1 1 1
   1 1 1 1 1 1 1 1 12 12 12 1 1 1 1 1 128 128 128 0 0 0 0 0 5 4 4 0 4 3
   4 0 178 194 32 0 180 196 32 0 3 0 0 144 1 0 66 31 0 2 2 32 48 0 0 0
   0 0 0 0 0 0 0 0 0 0])

(def jiggly-song
  [208 0 0 255 0 240 113 7 114 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0
   0 0 0 0 0 0 0 0 208 208 0 0 0 0 0 0 65 72 0 0 0 0 0 0 0 0 0 0 0 0 0
   0 128 0 0 0 0 0 0 0 105 130 0 0 0 0 0 0 0 0 0 0 0 0 0 0 17 16 0 0 0
   0 0 0 65 81 0 0 0 0 0 0 58 57 0 0 0 0 0 0 8 5 0 0 0 0 0 0 0 0 0 0 0
   0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0
   0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 7 3 1 1 1 1 1 1 1
   1 1 1 1 1 1 1 13 12 1 1 1 1 1 1 128 0 0 0 88 152 0 0 4 4 4 0 4 4 3 0
   103 167 32 0 180 196 32 0 0 0 0 144 1 0 66 28 0 31 31 224 255 0 0 0
   0 0 0 0 0 0 0 0 0 0])

(def fight-gym-leader-song
  [234 0 0 255 0 0 92 43 94 112 95 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0
   0 0 0 0 0 0 0 0 234 234 234 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0
   0 192 192 0 0 0 0 0 0 0 0 0 0 0 0 0 0 7 7 0 0 0 0 0 0 33 17 17 0 0 0
   0 0 68 82 0 0 0 0 0 0 89 90 57 0 0 0 0 0 8 8 0 0 0 0 0 0 0 0 0 0 0 0
   0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0
   0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 4 4 4 1 1 1 1 1 1 1
   1 1 1 1 1 1 12 12 12 1 1 1 1 1 192 192 192 0 88 152 0 0 7 5 4 0 4 4
   3 0 179 195 32 0 180 196 32 0 3 0 0 104 1 0 65 176 0 8 8 224 255 0 0
   0 0 0 0 0 0 0 0 0 0 0])


(def fight-gym-leader-song-simp
  [234 0 0 255 0 0 92 43 94 112 95 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0
   0 0 0 0 0 0 0 0 234 234 234 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0
   0 192 192 0 0 0 0 0 0 0 0 0 0 0 0 0 0 7 7 0 0 0 0 0 0 33 17 17 0 0 0
   0 0 68 82 0 0 0 0 0 0 89 90 57 0 0 0 0 0 8 8 0 0 0 0 0 0 0 0 0 0 0 0
   0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0
   0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 4 4 4 1 1 1 1 1 1 1
   1 1 1 1 1 1 12 12 12 1 1 1 1 1 192 192 192 0 88 152 0 0 7 5 4 0 4 4
   3 0 179 195 32 0 180 196 32 0 3 0 0 104 1 0 65 176 0 8 8 224 255 0 0
   0 0 0 0 0 0 0 0 0 0 0])

(defn change-music
  ([#^SaveState state start length song]

     (set-memory-range state (+ 0xC001 start)
                       (take length (drop start song))))
  ([start length song]
     (continue! (change-music (read-state "rlm-viridian-forest")
                                         start length song))))
   
  

;;0 0x90) blend between both, with end theme as well

;; 0x05 0xEA is the limit to completely change song to celadon from
;; viridian forest

;; 0x05 0xEA is also sufficient to change music to celadon from
;; a battle in viridian forest

(def minimal-celadon-song
  (subvec celadon-song 0x05 0xEA))

;; 0xC006 -- appears to be the song note counter.
;; 0cC007 -- 